'Frequency counter using RP2040 PIO

'PIO machine 1.0 generates a progr. gate signal on GP0 (side set pin GP0)
'PIO machine 1.1 uses the gate signal to count GP1 pulses

'PIO program and configuration ------------------------------------------------------

'PIO 1.0 (gate)
'adress data  mnemonics                   comment
' 0 E081 set pindir 00001                 'set GP0 to output
' 1 91A0  pull block .side(1)             'pull data from FIFO into OSR when available
' 2 B027  mov(x,osr) .side(1)             'load gate time (in clock pulses) from osr
' 3 0043  jmp(x_dec, this line) .side(0)  '1 cycle count down + gate low
' 4 1001  jmp 01 .side(1)                 'loop back

'PIO 1.1 (count)
' 5 A02B  mov(x,-1) .side(0)          'x = -1
' 6 2080  wait(1,pin,0) .side(0)      'sync
' 7 2000  wait(0,pin,0) .side(0)      'detect falling edge of gate
' 8 3001  wait(0,pin,1) .side(1)      'wait for risig edge pulse
' 9 3081  wait(1,pin,1) .side(1)      'wait for falling edge pulse
' A 00CC  jmp(pin, pc=0x0c) .side(0)  'continue counting ?
' B 0048  jmp(x_dec, pc=0x08) .side(0)'yes, count next pulse
' C A0C9  mov(isr,-x) .side(0)        'copy counter to isr
' D 8000  push noblock .side(0)       'get value to FIFO
' E 0005  jmp 05 .side(0)             'prepare for next gate pulse

'this is above program in data statements
dim a%(7)=(&h0043B02791A0E081,&h20002080A02B1001,&h004800CC30813001,&h58000A0C9,0,0,0,0)


'SETUP code
setpin gp0,pio1 'gate signal
setpin gp1,pio1 'frequency counter input
setpin gp2,pio1 'no signal output used to display input signal missing

'program above code in the chip in PIO 1
pio program 1,a%()


'PIO configuration -------------------------------------------------------------------------

'PIO 1.0
f0=1e6                            '1 MHz frequency gate resolution
e0=pio(execctrl 0,0,&h1f)         'default value
p0=Pio(pinctrl 1,1,,,gp0,gp0,)    'GP0 side set, GP0 SET (for pindirs)
pio init machine 1,0,f0,p0,e0,0,0 'start from 0x00

'PIO 1.1
f1=63e6                           '63 MHz frequency
e1=pio(execctrl 0,5,&h1f)         'default value for this machine
p1=Pio(pinctrl 1,1,,gp0,gp2,gp2,) 'GP0 base for inputs, GP2 out for status
pio init machine 1,1,f1,p1,e1,0,5 'start from 0x05


'Start both PIO sequencers
pio execute 1,1,&hE081            'set GP2 output
PIO start 1,0 'this will wait for data to arrive in FIFO, then generate 1 gate
PIO start 1,1 'this will start counter, waiting on adress 7 for GP0 to become low



'MMBASIC MAIN CODE --------------------------------------------------------------------------

'generate a test frequency on GP3, for testing of the counter
setpin gp3,pwm
pwm 1,15000000,,50      'change 15000000 to test different frequencies.
                        '15MHz PLL generates 15.75MHz dues to divider settings inside the PLL


'variables and constants used in the counter
dim r%(3),c%            'define variables, r%() only used to empty fifo
gate_time = 0.1         'seconds
gate_clocks% = gate_time*f0 - 1      'for 1MHz PIO gate clock as integer (n counts (n-1...0))


'this loop does the actual counting
do

 if pin(gp2)=1 then

  print "no input signal"
  pause 200

 else

  'put gate time in PIO 1.0 OSR (gate time is specified in PIO cycles, could be 1MHz, 10MHz..)
  pio read 1,1,4,r%()             'empty fifo from rubbish (only needed at start)
  pio write 1,0,1,gate_clocks%    'this starts the gate pulse generator, single pulse

  'wait gate time + few ms for printing data
  pause (1000*gate_time + 200)

  'get FIFO value
  pio read 1,1,1,c%        'read value from FIFO (there is only 1 value in it)
  c%=c%/gate_time          'convert or Hz

  'print frequency value
  print "GP1 frequency = ";c%;" Hz"
 end if

loop